#	ZXMedia: ZX Spectrum file format manipulation library
#	Copyright (C) 2004 Matthew Westcott
#	
#	This file is part of ZXMedia.
#	
#	ZXMedia is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License version 2, as
#	published by the Free Software Foundation.
#	
#	ZXMedia is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#	
#	You should have received a copy of the GNU General Public License
#	along with ZXMedia; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Contact address: Matthew Westcott, 14 Daisy Hill Drive, Adlington,
#	Chorley, Lancs. PR6 9NE, United Kingdom

module ZXMedia
	class ChecksummedIO
		# This class is a wrapper for an IO object which calculates
		# a simple sum of the bytes it writes.
		
		attr_reader :checksum
		attr_writer :checksum
		
		def initialize(io)
			@checksum = 0
			@io = io
		end

		def <<(obj)
				obj_string = obj.to_s
				obj_string.each_byte { |b|
					@checksum += b
				}
				@io << obj_string
		end

		def print(*objects)
			(objects.empty? ? [$_] : objects).each { |obj|
				obj.each_byte { |b|
					@checksum += b
				}
			}
			$\.each_byte { |b|
				@checksum += b
			} unless $\.nil?
			@io.print(*objects)
		end

		def printf(format, *objects)
			sprintf(format, *objects).each_byte { |b|
				@checksum += b
			}
			@io.sprintf(format, *objects)
		end

		def putc(obj)
			if obj.is_a? Fixnum
				@checksum += obj
			else
				@checksum += obj[0]
			end
			@io.putc(obj)
		end

		def puts(*objects)
			raise "puts not implemented for ChecksummedIO (because I'm lazy)"
		end

		def syswrite(str)
			str.each_byte { |b|
				@checksum += b
			}
			@io.syswrite(str)
		end

		def write(obj)
			obj_string = obj.to_s
			obj_string.each_byte { |b|
				@checksum += b
			}
			@io.write(obj_string)
		end
		
		# catch-all for methods that don't affect the checksum
		def method_missing( name, *args )
			@io.__send__( name, *args )
		end

	end # ChecksummedIO
end # ZXMedia