#	ZXMedia: ZX Spectrum file format manipulation library
#	Copyright (C) 2004 Matthew Westcott
#	
#	This file is part of ZXMedia.
#	
#	ZXMedia is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License version 2, as
#	published by the Free Software Foundation.
#	
#	ZXMedia is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#	
#	You should have received a copy of the GNU General Public License
#	along with ZXMedia; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Contact address: Matthew Westcott, 14 Daisy Hill Drive, Adlington,
#	Chorley, Lancs. PR6 9NE, United Kingdom

module ZXMedia
	class CodeBlock < Block
		# This class represents a code ('Bytes:') block.
		
		attr_reader(:filename, :start_address, :data, :tape_representation, :trdos_representation)
		attr_writer(:tape_representation, :trdos_representation)

		def initialize(filename, data, start_address)
			@tape_representation = nil
			@trdos_representation = nil

			@filename = filename
			@data = data
			@start_address = start_address
		end

		def filename=(f)
			@filename = f
			@trdos_representation.filename = filename unless (@trdos_representation.nil?)
		end
		
		def start_address=(address)
			@start_address = address
			@trdos_representation.filename = filename unless (@trdos_representation.nil?)
		end

		def block_length
			@data.size
		end

		def to_a
			@data
		end

		def trdos_block
			@trdos_representation = self.canonical_trdos_representation if (@trdos_representation.nil?)
			@trdos_representation.trdos_block
		end

		def write_tap_block(io)
			@tape_representation = self.canonical_tape_representation if (@tape_representation.nil?)
			@tape_representation.each { |block|
				block.write_tap_block(io)
			}
		end

		def write_tzx_block(io)
			@tape_representation = self.canonical_tape_representation if (@tape_representation.nil?)
			@tape_representation.each { |block|
				block.write_tzx_block(io)
			}
		end

		def canonical_tape_representation
			[BytesHeaderTapeBlock.new(self), DataTapeBlock.new(self)]
		end

		def canonical_trdos_representation
			TRDOSCodeBlock.new(@filename, @start_address, self)
		end

		def CodeBlock.from_tape(subblocks)
			# find the header and datablock
			header = nil
			datablock = nil
			subblocks.each { |block|
				if block.is_a? BytesHeaderTapeBlock
					header = block
				end
				if block.is_a? DataTapeBlock
					datablock = block
					break
				end unless header.nil?
			}
			raise "BytesHeaderTapeBlock missing from tape block list" if header.nil?
			raise "DataTapeBlock missing from tape block list" if datablock.nil?

			code_block = CodeBlock.new(header.filename, datablock.data, header.start_address)
			code_block.tape_representation = subblocks
			header.datasource = code_block
			datablock.datasource = code_block
			code_block
		end

		def CodeBlock.from_trdos(trdos)
			raise "CodeBlock.from_trdos called on a non-TRDOSCodeBlock block" unless trdos.is_a? TRDOSCodeBlock
			code_block = CodeBlock.new(trdos.filename, trdos.data, trdos.start_address)
			code_block.trdos_representation = trdos
			trdos.data = code_block
			code_block
		end
		
		def CodeBlock.can_promote?(block)
			block.is_a? TRDOSCodeBlock
		end

		def CodeBlock.promote(block)
			code_block = CodeBlock.new(block.filename, block.data, block.start_address)
			code_block.trdos_representation = block
			block.datasource = code_block
			code_block
		end

		def to_s
			"Bytes: \"#{self.filename}\" CODE #{self.start_address},#{self.data.length} (#{self.data.length} bytes)"
		end

	end # CodeBlock
end # ZXMedia