#	ZXMedia: ZX Spectrum file format manipulation library
#	Copyright (C) 2004 Matthew Westcott
#	
#	This file is part of ZXMedia.
#	
#	ZXMedia is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License version 2, as
#	published by the Free Software Foundation.
#	
#	ZXMedia is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#	
#	You should have received a copy of the GNU General Public License
#	along with ZXMedia; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Contact address: Matthew Westcott, 14 Daisy Hill Drive, Adlington,
#	Chorley, Lancs. PR6 9NE, United Kingdom

module ZXMedia
	class ProgramHeaderTapeBlock < Block
		# This class represents a 17-byte header for a Program: block on a tape.
		
		attr_reader(:datasource, :header_tape_block)
		attr_writer(:datasource, :header_tape_block)

		def initialize(datasource_or_filename, block_length = nil, basic_length = nil, autostart_line = nil)
			if datasource_or_filename.is_a? String
				# filename supplied - other fields are required
				raise "block_length not specified for ProgramHeaderTapeBlock" if block_length.nil?
				raise "basic_length not specified for ProgramHeaderTapeBlock" if basic_length.nil?
				@datasource = nil
				self.filename = datasource_or_filename
				self.block_length = block_length
				self.basic_length = basic_length
				self.autostart_line = autostart_line
			else
				# datasource supplied
				@datasource = datasource_or_filename
			end
			self.header_tape_block = HeaderTapeBlock.new(self)
		end
		
		def filename
			if @datasource.nil?
				@filename
			else
				@datasource.filename
			end
		end

		def filename=(f)
			raise "Cannot assign filename to this block directly" unless @datasource.nil?
			# trim / space-pad filename to 10 characters
			@filename = f.ljust(10)[0,10]
		end

		def block_length
			if @datasource.nil?
				@block_length
			else
				@datasource.block_length
			end
		end

		def block_length=(block_length)
			raise "Cannot assign block_length to this block directly" unless @datasource.nil?
			@block_length = block_length
		end

		def basic_length
			if @datasource.nil?
				@basic_length
			else
				@datasource.basic_length
			end
		end

		def basic_length=(basic_length)
			raise "Cannot assign basic_length to this block directly" unless @datasource.nil?
			@basic_length = basic_length
		end

		def autostart_line
			if @datasource.nil?
				@autostart_line
			else
				@datasource.autostart_line
			end
		end

		def autostart_line=(line)
			raise "Cannot assign autostart_line to this block directly" unless @datasource.nil?
			@autostart_line = (1..9999) === line ? line : nil
		end

		def header_type_id
			0
		end

		def header_tape_extra_data
			if self.autostart_line.nil?
				autostart_line = 32768
			else
				autostart_line = self.autostart_line
			end
			Common.word_little_endian(autostart_line) + Common.word_little_endian(self.basic_length)
		end

		def to_s
			"Header for program \'#{self.filename}\'" + (self.autostart_line.nil? ? "" : " LINE #{self.autostart_line}")
		end

		def write_tap_block(io)
			self.header_tape_block.write_tap_block(io)
		end
		
		def write_tzx_block(io)
			self.header_tape_block.write_tzx_block(io)
		end

		def ProgramHeaderTapeBlock.can_promote?(block)
			if block.is_a? HeaderTapeBlock
				block.header_type_id == 0 && block.extra_data.size == 4
			else
				false
			end
		end

		def ProgramHeaderTapeBlock.promote(block)
			data = block.extra_data
			autostart_line = data[0] | (data[1] << 8)
			basic_length = data[2] | (data[3] << 8)
			phtb = ProgramHeaderTapeBlock.new(block.filename, block.block_length, basic_length, autostart_line)
			block.datasource = phtb
			phtb.header_tape_block = block
			phtb
		end

	end # ProgramHeaderTapeBlock
end # ZXMedia