#	ZXMedia: ZX Spectrum file format manipulation library
#	Copyright (C) 2004 Matthew Westcott
#	
#	This file is part of ZXMedia.
#	
#	ZXMedia is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License version 2, as
#	published by the Free Software Foundation.
#	
#	ZXMedia is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#	
#	You should have received a copy of the GNU General Public License
#	along with ZXMedia; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Contact address: Matthew Westcott, 14 Daisy Hill Drive, Adlington,
#	Chorley, Lancs. PR6 9NE, United Kingdom

module ZXMedia
	def ZXMedia.read_tzx(io)
		medium = Medium.new
		signature = io.read(8)
		if signature != "ZXTape!\x1a"
			raise "File is not a TZX file"
		end
		begin
			major_version = io.readchar
			minor_version = io.readchar
			if major_version != 1
				$stderr << "Warning: TZX file is version #{major_version}.#{minor_version}, but only 1.* is supported;\nwill try to read file anyway"
			end
			until io.eof?
				block_id = io.readchar
				case block_id
					when 0x10 # standard speed tape block
						pause_length = io.readchar + (io.readchar << 8)
						block_length = io.readchar + (io.readchar << 8)
						block_data = []
						block_length.times do
							block_data << io.readchar
						end
						medium.blocks << StandardSpeedTapeBlock.new(block_data, pause_length)
					when 0x11 # turbo loading tape block
						pilot_pulse_length = io.readchar + (io.readchar << 8)
						sync_first_pulse_length = io.readchar + (io.readchar << 8)
						sync_second_pulse_length = io.readchar + (io.readchar << 8)
						zero_bit_pulse_length = io.readchar + (io.readchar << 8)
						one_bit_pulse_length = io.readchar + (io.readchar << 8)
						pilot_tone_length = io.readchar + (io.readchar << 8)
						trailing_bits = io.readchar
						trailing_pause_length = io.readchar + (io.readchar << 8)
						block_length = io.readchar + (io.readchar << 8) + (io.readchar << 16)
						block_data = []
						block_length.times do
							block_data << io.readchar
						end
						medium.blocks << TurboTapeBlock.new(block_data, pilot_pulse_length, sync_first_pulse_length, sync_second_pulse_length, zero_bit_pulse_length, one_bit_pulse_length, pilot_tone_length, trailing_bits, trailing_pause_length)
					when 0x12 # pure tone
						pulse_length = io.readchar + (io.readchar << 8)
						pulse_count = io.readchar + (io.readchar << 8)
						medium.blocks << PureToneTapeBlock(pulse_length, pulse_count)
					when 0x13 # sequence of pulses
						pulse_count = io.readchar
						pulses = []
						pulse_count.times do
							pulses << io.readchar + (io.readchar << 8)
						end
						medium.blocks << PulseSequenceTapeBlock(pulses)
					when 0x14 # pure data
						zero_bit_pulse_length = io.readchar + (io.readchar << 8)
						one_bit_pulse_length = io.readchar + (io.readchar << 8)
						trailing_bits = io.readchar
						trailing_pause_length = io.readchar + (io.readchar << 8)
						block_length = io.readchar + (io.readchar << 8) + (io.readchar << 16)
						block_data = []
						block_length.times do
							block_data << io.readchar
						end
						medium.blocks << PureDataTapeBlock.new(block_data, zero_bit_pulse_length, one_bit_pulse_length, trailing_bits, trailing_pause_length)
					when 0x15 # direct recorded data
						t_states_per_sample = io.readchar + (io.readchar << 8)
						trailing_pause_length = io.readchar + (io.readchar << 8)
						trailing_bits = io.readchar
						block_length = io.readchar + (io.readchar << 8) + (io.readchar << 16)
						block_data = []
						block_length.times do
							block_data << io.readchar
						end
						medium.blocks << DirectRecordedTapeBlock.new(data, t_states_per_sample, trailing_pause_length, trailing_bits)
					# TODO: Handle C64 blocks 0x16 and 0x17. Currently treated as unknown blocks,
					# respecting the TZX extension rule
					when 0x20 # pause / stop tape
						length = io.readchar + (io.readchar << 8)
						medium.blocks << PauseTapeBlock.new(length)
					when 0x21 # group start
						length = io.readchar
						name = io.read(length)
						medium.blocks << GroupStartTapeBlock.new(name)
					when 0x22 # group end
						medium.blocks << GroupEndTapeBlock.new
					when 0x23 # jump to block
						offset = io.readchar + (io.readchar << 8)
						if offset > 0x7fff
							offset -= 0x10000
						end
						medium.blocks << JumpTapeBlock.new(offset)
					when 0x24 # loop start
						count = io.readchar + (io.readchar << 8)
						medium.blocks << LoopStartTapeBlock.new(count)
					when 0x25 # loop end
						medium.blocks << LoopEndTapeBlock.new
					when 0x26 # call sequence
						length = io.readchar + (io.readchar << 8)
						calls = []
						length.times do
							calls << io.readchar + (io.readchar << 8)
						end
						medium.blocks << CallSequenceTapeBlock.new(calls)
					when 0x27 # call return
						medium.blocks << CallReturnTapeBlock.new
					when 0x28 # selector menu
						length = io.readchar + (io.readchar << 8)
						data = []
						length.times do
							data << io.readchar
						end
						medium.blocks << SelectTapeBlock.new(data)
					# TODO: Handle block 0x2A, 'stop tape if in 48k mode'.
					# Currently treated as an unknown block,
					# respecting the TZX extension rule
					when 0x30 # text description
						length = io.readchar
						text = io.read(length)
						medium.blocks << TextDescriptionTapeBlock.new(text)
					when 0x31 # message display
						show_time = io.readchar
						length = io.readchar
						text = io.read(length)
						medium.blocks << MessageTapeBlock.new(text, show_time)						
					when 0x32 # archive info
						length = io.readchar + (io.readchar << 8)
						data = []
						length.times do
							data << io.readchar
						end
						medium.blocks << ArchiveInfoTapeBlock.new(data)
					when 0x33 # hardware type
						length = io.readchar
						capabilities = []
						length.times do
							hardware_type = io.readchar
							hardware_id = io.readchar
							support_status = io.readchar
							capabilities << HardwareCapability.new(hardware_type, hardware_id, support_status)
						end
						medium.blocks << HardwareTypeTapeBlock.new(capabilities)
					when 0x34 # emulation info
						byte1 = io.readchar
						r_register = byte1 & 0x01 != 0
						ldir = byte1 & 0x02 != 0
						high_res_color = byte1 & 0x04 != 0
						video_sync = (byte1 & 0x18) >> 3
						fast_loading = byte1 & 0x20 != 0
						border = byte1 & 0x40 != 0
						screen_refresh = byte1 & 0x80 != 0
						byte2 = io.readchar
						auto_start_tape = byte2 & 0x01 != 0
						auto_load = byte2 & 0x02 != 0
						reserved_bits = byte2 >> 2
						screen_refresh_delay = io.readchar
						interrupt_frequency = io.readchar + (io.readchar << 8)
						reserved = io.readchar + (io.readchar << 8) + (io.readchar << 16)
						medium.blocks << EmulationInfoTapeBlock(r_register, ldir, high_res_color, video_sync, fast_loading, border, screen_refresh, auto_start_tape, auto_load, screen_refresh_delay, interrupt_frequency, reserved_bits, reserved)
					when 0x35 # custom info
						id = io.read(10)
						data_length = io.readchar + (io.readchar << 8) + (io.readchar << 16) + (io.readchar << 24)
						data = []
						data_length.times do
							data << io.readchar
						end
						medium.blocks << CustomInfoTapeBlock(id, data)
					when 0x40 # snapshot
						snapshot_type = io.readchar
						length = io.readchar + (io.readchar << 8) + (io.readchar << 16)
						data = []
						length.times do
							data << io.readchar
						end
						medium.blocks << SnapshotTapeBlock(snapshot_type, data)
					when 0x5a # concatenated 'ZXTape!' header
						# as per the TZX spec, we'll drop these blocks even though
						# it means sacrificing some (redundant) detail of the original
						# block
						io.read(9) # skip 9 bytes
					else
						$stderr << "Warning: Encountered unknown TZX block type #{block_id}, assuming TZX extension rule\n"
						data_length = io.readchar + (io.readchar << 8) + (io.readchar << 16) + (io.readchar << 24)
						data = []
						length.times do
							data << io.readchar
						end
						medium.blocks << UnknownTapeBlock(block_id, data)
				end
			end
		rescue EOFError
			$stderr << "Warning: Unexpected end of file, discarding incomplete final block\n"
		end

		medium.consolidate

	end # read_tzx
end #ZXMedia
