#	ZXMedia: ZX Spectrum file format manipulation library
#	Copyright (C) 2004 Matthew Westcott
#	
#	This file is part of ZXMedia.
#	
#	ZXMedia is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License version 2, as
#	published by the Free Software Foundation.
#	
#	ZXMedia is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#	
#	You should have received a copy of the GNU General Public License
#	along with ZXMedia; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Contact address: Matthew Westcott, 14 Daisy Hill Drive, Adlington,
#	Chorley, Lancs. PR6 9NE, United Kingdom

module ZXMedia
	class TRDOSBASICBlock < Block
		# This class represents a BASIC (extension B) file on a TR-DOS (Betadisk) filesystem.

		attr_reader(:shadow_data, :datasource, :trdos_block)
		attr_writer(:shadow_data, :datasource, :trdos_block)
		
		def initialize(data, filename = nil, basic_length = nil, autostart_line = nil, shadow_data = [])
			# data can be either an array or an external object to use as a datasource
			if data.is_a? Array
				# array supplied - need other fields to be supplied too
				raise "filename not specified for TRDOSBASICBlock" if filename.nil?
				raise "basic_length not specified for TRDOSBASICBlock" if basic_length.nil?

				@data = data
				@datasource = nil
				self.filename = filename
				self.autostart_line = autostart_line
				self.basic_length = basic_length
			else
				@data = []
				@datasource = data
			end
			@shadow_data = shadow_data
			@trdos_block = TRDOSBlock.new(self)
		end

		def filename
			if @datasource.nil?
				@filename
			else
				@datasource.filename.ljust(8)[0,8]
			end
		end

		def filename=(name)
			raise "Cannot assign filename to this block directly" unless @datasource.nil?
			@filename = name.ljust(8)[0,8]
		end

		def data
			if @datasource.nil?
				@data
			else
				@datasource.data
			end
		end

		def data=(x)
			raise "Cannot assign data to this block directly" unless @datasource.nil?
			@data = x
		end

		def extension
			"B"
		end

		def basic_length
			if @datasource.nil?
				@basic_length
			else
				@datasource.basic_length
			end
		end
		
		def basic_length=(x)
			raise "Cannot assign basic_length to this block directly" unless @datasource.nil?
			@basic_length = x
		end

		def autostart_line
			if @datasource.nil?
				@autostart_line
			else
				@datasource.autostart_line
			end
		end
		
		def autostart_line=(x)
			raise "Cannot assign autostart_line to this block directly" unless @datasource.nil?
			@autostart_line = x
		end

		def trdos_param1
			self.data.size
		end

		def trdos_param2
			self.basic_length
		end

		def trdos_block_data
			line_number = self.autostart_line.nil? ? 0 : self.autostart_line
			basic_part = self.data + [0x80, 0xAA] + Common.word_little_endian(line_number)
			basic_sector_count = (basic_part.size / 256.0).ceil
			basic_padding = [0] * (basic_sector_count * 256 - basic_part.size)
			shadow_part = self.shadow_data
			shadow_sector_count = (shadow_part.size / 256.0).ceil
			shadow_padding = [0] * (shadow_sector_count * 256 - shadow_part.size)

			basic_part + basic_padding + shadow_part + shadow_padding
		end

		def to_s
			"TR-DOS file \"#{self.filename}.B\" " + (self.autostart_line.nil? ? "" : "LINE #{self.autostart_line} ") + "(#{self.data.size} bytes)"
		end

		def TRDOSBASICBlock.can_promote?(block)
			if block.is_a? TRDOSBlock
				block.extension == 'B'
			else
				false
			end
		end

		def TRDOSBASICBlock.promote(block)
			visible_length = block.param1
			basic_length = block.param2
			visible_sector_count = ((visible_length + 4) / 256.0).ceil
			data = block.data
			visible_data = data[0, visible_length]
			tag = data[visible_length, 4]
			magic_number = tag[0] | (tag[1] << 8)
			autostart_line = tag[2] | (tag[3] << 8)
			$stderr << "Warning: magic number 0xAA80 not found after TR-DOS BASIC block data\n" unless magic_number == 0xAA80
			autostart_line = nil unless (1..9999) === autostart_line
			shadow_data = data[(visible_sector_count * 256)..(data.size - 1)]
			tbb = TRDOSBASICBlock.new(visible_data, block.filename, basic_length, autostart_line, shadow_data)
			block.datasource = tbb
			tbb.trdos_block = block
			tbb
		end

		def TRDOSBASICBlock.promotion_targets
			[BASICBlock]
		end

	end # TRDOSBASICBlock
end # ZXMedia